/**
 * @file g722_codec.h
 * @author Phil Schatzmann
 * @brief Include for encoder and decoder which supports C++
 * @version 0.1
 * @date 2022-05-08
 *
 * @copyright Copyright (c) 2022
 *
 */

#pragma once

#include "g722_encoder.h"
#include "g722_decoder.h"
